#ifndef FEMSOLVER_H
#define FEMSOLVER_H

#include "fem.h"
#include "femDOF.h"
#include "femTimer.h"
#include <Eigen/Dense>
#include <Eigen/Sparse>

namespace fem
{

/// FEM solver

class FEM_API Solver
{
public:
#ifndef SWIG
    Problem &pbl;         ///< problem data
    Partition *partition; ///< partition of the dofs
    Part *part_Mfree;
    Part *part_Mfixed;

    DOFs dofs;
    DOFs bcs;

    Eigen::ArrayXXi dofFixed; ///< is a dof fixed or free?
    Eigen::ArrayXXi dofPart;  ///< in which part is a given dof of a given node?
    Eigen::ArrayXXi dofIdx;
    std::vector<size_t> partSizes;

    std::vector<std::vector<Eigen::SparseMatrix<double> *>> K;
    std::vector<Eigen::VectorXd *> f;
    std::vector<Eigen::VectorXd *> X;

    std::map<std::string, Timer> timers;

#endif
    std::string linear_solver;
    double int_energy;

public:
    Solver(Problem &_pbl);
    ~Solver();

    void solve();

#ifndef SWIG
    void compute_sig_epl(std::vector<size_t> &etags_all,
                         std::vector<std::vector<double>> &sigma_all,
                         std::vector<std::vector<double>> &epsilon_all,
                         std::map<size_t, std::vector<Eigen::VectorXd>> &sig_by_node,
                         std::map<size_t, std::vector<Eigen::VectorXd>> &eps_by_node);

    friend FEM_API std::ostream &operator<<(std::ostream &out, Solver const &obj);
#endif

private:
    void create_partition();
    void fill_partition();
    void fill_rhs();
    void fill_lhs();
    void apply_bc();
    void solve_static();
};

} // namespace fem

#endif // FEMSOLVER_H
